class FullcalendarTimeline {

  // Assumption:
  // Each timeline element id is unique <div id='timelineElementId'>, and associated form fields
  // are encapsulated by a unique id (e.g. <fieldset id='formFieldsId'>)
  // (TBD: Add to documentation)

  constructor(formFieldsId, timelineElementId) {
    this._initialLocaleCode = navigator.language.toLowerCase(); // e.g. 'en-gb' (navigator.languages might be better)
    this._initialTimezone = Intl.DateTimeFormat().resolvedOptions().timeZone; // e.g. 'Europe/London'

    this._formFieldsId = formFieldsId;
    this._formFieldsElement = $('#' + formFieldsId);
    this._formFieldsWrapper = this._formFieldsElement;

    this._timelineElementId = timelineElementId;
    this._timelineElement = $('#' + timelineElementId);
    this._timelineViewWrapper = this._timelineElement.closest('#timeline-view');

    this._fullcalendarObject = new FullCalendar.Calendar(this._timelineElement[0], {
      schedulerLicenseKey: 'GPL-My-Project-Is-Open-Source',
      plugins: ['interaction', 'resourceTimeline'], //, 'rrule'],
      defaultDate: new Date().setMinutes(0), // now
      timeZone: 'local',
      editable: true, // enable draggable events
      aspectRatio: 1.8, // default: 1.35
      scrollTime: '00:00', // undo default 6am scrollTime

      header: {
        left: 'promptCourse,promptTrainer,promptRoom today prev,next',
        center: 'title',
        right: 'zoomIn,zoomOut'
      },

      customButtons: {
        // zoomIn: {
        //   text: '<',
        // },
        // zoomOut: {
        //   text: '>',
        // },
        promptCourse: {
          text: '+Course',
          click: function () {
            var title = prompt('Course name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        },
        promptTrainer: {
          text: '+Trainer',
          click: function () {
            var title = prompt('Trainer name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        },
        promptRoom: {
          text: '+Room',
          click: function () {
            var title = prompt('Room name');
            if (title) {
              $('#fullcalendar_scheduler').fullCalendar(
                'addResource',
                { title: title },
                true // scroll to the new resource?
              );
            }
          }
        }
      },

      //defaultView: 'timeline3Days',
      //defaultView: 'timeline1Day',
      //defaultView: 'timeline',
      defaultView: 'resourceTimelineDay',

      // visibleRange: function (currentDate) {
      //   return {
      //     start: currentDate, //window.visibleRangeStartDate,
      //     end: currentDate.clone().add(3, 'days') //window.visibleRangeEndDate
      //   };
      // },

      // validRange: function(nowDate) {
      //   return {
      //     start: dateAdd(nowDate, -3, 'days'), //window.visibleRangeStartDate,
      //     end: dateAdd(nowDate, 3, 'days') //window.visibleRangeEndDate
      //   };
      // },

      // views: {
      //   resourceTimeline2Days: {
      //     type: 'resourceTimeline',
      //     duration: { days: 2 }
      //   },
      //   resourceTimeline3Days: {
      //     type: 'resourceTimeline',
      //     duration: { days: 3 }
      //   },
      //   resourceTimeline: {
      //     // options apply to resourceTimeline views (resourceTimelineDay, resourceTimelineWeek, resourceTimelineMonth)
      //     visibleRange: function (currentDate) {
      //       return {
      //         start: window.visibleRangeStartDate,
      //         end: window.visibleRangeEndDate
      //       };
      //     },
      //   },
      //   dayGrid: {
      //     // options apply to dayGridDay, dayGridWeek and dayGridMonth views
      //   },
      //   timeGrid: {
      //     // options apply to timeGridDay and timeGridWeek views
      //   },
      //   week: {
      //     // options apply to dayGridWeek and timeGridWeek views
      //   },
      //   day: {
      //     // options apply to dayGridDay and timeGridDay views
      //   }

      // },

      resourceAreaWidth: '20%',
      resourceGroupField: 'resourceType',

      // If refetchResourcesOnNavigate is set to true, the network request will
      // be made with additional parameters: the start/end date of the newly
      // visible window of time, as well as the calendar’s timezone
      //
      // example request: /resource_planner/resources.json?start=2017-12-01&end=2017-01-12&timezone=UTC

      refetchResourcesOnNavigate: true,
      resources: {
        url: '/resource_planner/resources.json',
        type: 'GET',
      },

      eventSources: getEventSources(),

      // Callbacks

      resourceRender: function (renderInfo) {
        $(renderInfo.el).on('click', function () {
          resourceSelectedHandler(renderInfo);
        });
      },

      viewSkeletonRender: function (info) {
        // View’s non-date-related DOM structure has been rendered.
        // This callback will get triggered when the initial view renders or when
        // the user changes the view, but before the datesRender callback fires.
      },

      datesRender: function (info) {
        // All date/time cells have been rendered.
        // This triggers after viewSkeletonRender callback but before the eventRender callbacks.

        // attachTimelineManipulationHandlers();
        // renderScrollBar();
        // initialiseHighlighterbarTooltip();

        // if (typeof window.eventsArray !== 'undefined') {
        //   renderEvents(window.eventsArray, window.StartingDate, window.endingDate);
        // }

        // renderSelectedResources();
      },

      eventRender: function (info) {
      },

      eventDrop: function (event, delta, revertFunc) {
        if (!confirm(event.title + " was changed to " + event.start.format() + "\nAre you sure about this change?")) {
          revertFunc();
        }
      },

    }); // end new FullCalendar.Calendar

  } // end constructor

  // Getters/Setters
  get fullcalendarObject() {
    return this._fullcalendarObject;
  }

  get formFieldsWrapper() {
    return this._formFieldsElement;
  }

  get timelineElement() {
    return this._timelineElement;
  }

  get timelineViewWrapper() {
    return this._timelineViewWrapper;
  }

  get fullcalendarTimelineTimelineManipulator() {
    return this._fullcalendarTimelineTimelineManipulator;
  }

  get fullcalendarTimelineResourceManipulator() {
    return this._fullcalendarTimelineResourceManipulator;
  }

  get fullcalendarTimelineHighlighterBarManipulator() {
    return this._fullcalendarTimelineHighlighterBarManipulator;
  }

  // Initialise and render
  render() {
    this._fullcalendarObject.render();

    // Initialise the locale selector, etc.
    // - zoom control is in FullcalendarTimelineTimelineManipulator
    // - form fields & the highlighter-bar are in FullcalendarTimelineHighlighterBarManipulator
    // - resource arbitration is in FullcalendarTimelineResourceManipulator
    this.initialiseLocaleSelector();
    this.setLocaleCode(this._initialLocaleCode);

    // // Helper classes (consider making these modules of this class)
    this._fullcalendarTimelineTimelineManipulator = new FullcalendarTimelineTimelineManipulator(this);
    this._fullcalendarTimelineTimelineManipulator.render();

    this._fullcalendarTimelineHighlighterBarManipulator = new FullcalendarTimelineHighlighterBarManipulator(this);
    this._fullcalendarTimelineHighlighterBarManipulator.render();

    this._fullcalendarTimelineResourceManipulator = new FullcalendarTimelineResourceManipulator(this);
    this._fullcalendarTimelineResourceManipulator.render();
  }

  // Iniialise associated timeline view control elements
  initialiseLocaleSelector() {
    // Mechanism to pass (this) to the handlers
    var fullCalendarTimelineObject = this;

    var timelineViewWrapper = this._timelineViewWrapper;
    var localeSelector = timelineViewWrapper.find('#locale-selector');
    var initialLocaleCode = this._initialLocaleCode;

    // Initialise drop-down list of the locale selector
    $.each(FullCalendarLocalesAll, function (index) {
      localeSelector.append(
        $('<option/>')
          .attr('value', FullCalendarLocalesAll[index].code)
          .prop('selected', FullCalendarLocalesAll[index].code == initialLocaleCode)
          .text(FullCalendarLocalesAll[index].code)
      );
    });

    // Add event handler to the locale selector
    localeSelector.on('change', function () {
      if (this.value) {
        fullCalendarTimelineObject.setLocaleCode(this.value);
      }
    });
  }

  // Utility methods
  setLocaleCode(localeCode) {
    // TBD: only the datetimepicker in the associated form
    $.datetimepicker.setLocale(localeCode);
    this._fullcalendarObject.setOption('locale', localeCode);
  }

} // end class
