class FullcalendarTimelineTimelineManipulator {

  constructor(fullcalendarTimelineObject) {
    this._fullcalendarTimelineObject = fullcalendarTimelineObject;

    this._zoomInButton = fullcalendarTimelineObject.timelineViewWrapper.find('#zoom-in-button');
    this._zoomText = fullcalendarTimelineObject.timelineViewWrapper.find('#zoom-text');
    this._zoomScaleSelector = fullcalendarTimelineObject.timelineViewWrapper.find('#zoom-scale-selector');
    this._zoomOutButton = fullcalendarTimelineObject.timelineViewWrapper.find('#zoom-out-button');

    this._zoomScales = ['day', 'week', 'month'];
    this._currentDurationCount = 1;
    this._currentDurationScale = 'day';
    this._visibleRangeStartDate = new Date();
    this._visibleRangeEndDate = dateAdd(this._visibleRangeStartDate, this._currentDurationCount, this._currentDurationScale);
  } // end constructor

  // Getters/Setters
  get fullcalendarTimelineObject() {
    return this._fullcalendarTimelineObject;
  }

  // Initialise and render
  render() {
    // Mechanism to pass (this) to the handlers
    var fullcalendarTimelineTimelineManipulatorObject = this;

    // Initialise drop-down list of the 'zoom-scale-selector'
    $.each(this._zoomScales, function (index, text) {
      fullcalendarTimelineTimelineManipulatorObject._zoomScaleSelector.append(
        $('<option/>')
          .attr('value', text)
          .prop('selected', text == fullcalendarTimelineTimelineManipulatorObject._currentDurationScale)
          .text(text)
      );
    });

    // Add event handlers to the 'zoom-scale-selector'
    this._zoomScaleSelector.on('change', function () {
      var newScale = fullcalendarTimelineTimelineManipulatorObject._zoomScaleSelector.val();
      if (newScale != fullcalendarTimelineTimelineManipulatorObject._currentDurationScale) {
        fullcalendarTimelineTimelineManipulatorObject.updateTimelineDuration();
      }
    });

    this._zoomInButton.off('click').on('click', function () {
      fullcalendarTimelineTimelineManipulatorObject.timelineZoomHandler('+');
    });

    this._zoomOutButton.off('click').on('click', function () {
      fullcalendarTimelineTimelineManipulatorObject.timelineZoomHandler('-');
    });

    // Add a datetimepicker to the date view
    this.fullcalendarTimelineObject.timelineElement.find('.fc-header-toolbar .fc-center h2').datetimepicker({
      timepicker: false,
      format: 'Y-m-d',
      onChangeDateTime: function (dp, input) {
        fullcalendarTimelineTimelineManipulatorObject.updateTimelineDuration(input.val());
      },
    });
  }

  /**
   * Re-render timeline view with the specified date range
   *
   * @param direction   '+' = increase date range / '-' = decrease date range
   */
  timelineZoomHandler(direction) {
    switch (direction) {
      case '+':
        this._currentDurationCount = this._currentDurationCount + 1;
        break;
      case '-':
        this._currentDurationCount = this._currentDurationCount - 1;
        break;
    }

    this.updateTimelineDuration();
  }

  /**
   * Re-render timeline view with the specified date range
   *
   * Examples:
   * updateTimelineDuration()
   * - use the date fields from the form
   *
   * updateTimelineDuration('2019-11-21')
   * - use the supplied start_date + the 'currentDurationCount'
   *
   * updateTimelineDuration('2019-11-21','2019-11-29')
   * - override any form values
   *
   * @param start_date Start date
   * @param end_date   End date
   */
  updateTimelineDuration(start_date, end_date) {
    var conversionDivisor;

    this._currentDurationScale = this._zoomScaleSelector.val();

    switch (this._currentDurationScale) {
      case 'month':
        // TODO: Fix me
        // 1 month (28 days) = 2419200000 ms
        // 1 month (29 days) = 2505600000 ms
        // 1 month (30 days) = 2592000000 ms
        // 1 month (31 days) = 2678400000 ms
        conversionDivisor = (1000 * 60 * 60 * 24 * 30);
        break;
      case 'week':
        // 1 week = 604800000 ms
        conversionDivisor = (1000 * 60 * 60 * 24 * 7);
        break;
      case 'day':
      default:
        // 1 day =  86400000 ms
        conversionDivisor = (1000 * 60 * 60 * 24);
    }

    // TBD: An assumption is made that DTSTART is populated and formatted as a 10 character date
    //      Needs a fix here... (maybe, if it is empty, to populate it with today's date)
    if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
      start_date = $.trim( this.getFormField(formField().DTSTART).val() ).substring(0, 10);
    }

    // If we have no 'start_date', from any source, don't do anything
    if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
      return;
    }

    var startDate = new Date(start_date);

    if (typeof end_date === 'undefined' || end_date === null || end_date.length == 0) {
      end_date = dateAdd(startDate, this._currentDurationCount - 1, this._currentDurationScale).yyyymmdd();
    }

    var endDate = new Date(end_date);

    this._currentDurationCount = Math.round((endDate - startDate) / conversionDivisor) + 1;
    if (this._currentDurationCount < 1) { this._currentDurationCount = 1; }

    // Adjust timeline
    this._visibleRangeStartDate = new Date(startDate);
    this._visibleRangeEndDate = dateAdd(startDate, this._currentDurationCount, this._currentDurationScale);

    this.fullcalendarTimelineObject.fullcalendarObject.changeView('resourceTimeline', {
      start: this._visibleRangeStartDate,
      end: this._visibleRangeEndDate,
    });

    this.fullcalendarTimelineObject.fullcalendarObject.setOption('duration', { days: Math.abs(this._visibleRangeStartDate.diff(this._visibleRangeEndDate, 'day')) })

    this.fullcalendarTimelineObject.fullcalendarObject.gotoDate(this._visibleRangeStartDate);

    // Update zoom-text
    this._zoomText.text(this._currentDurationCount);
  }

  //
  // Helper methods
  getFormField(selector) {
    return this.fullcalendarTimelineObject._formFieldsElement.find(selector);
  }
} // end class
